#include "msdb.h"
#include "earthworm.h"
#include "mseedfilename.h"
#include "time_ew.h"
#include "trace_buf.h"
#include <math.h>
#include <string.h>

static void msdb_close(sqlite3 *pdb, sqlite3_stmt *select_stmt) {
  sqlite3_finalize(select_stmt);
  sqlite3_close(pdb);
}

static int msdb_bind_text(sqlite3_stmt *stmt, int index, const char *text) {
  return sqlite3_bind_text(stmt, index, text, strlen(text), NULL);
}

int msdb_get_filename(char *mseedfile, char *mseeddir, double t, char *stat,
                      char *chan, char *net, char *loc) {
  char start_timestr[DATESTR_MAX];
  char end_timestr[DATESTR_MAX];
  int flags = SQLITE_OPEN_READONLY;
  char *sql;
  int rc;
  sqlite3 *pdb;
  sqlite3_stmt *select_stmt = NULL;
  rc = sqlite3_open_v2(mseeddir, &pdb, flags, NULL);
  if (rc != SQLITE_OK) {
    return rc;
  }
  if (t > 0) {
    sql = "SELECT filename FROM tsindex WHERE network=? and station=? and "
          "location=? and channel=? and starttime<=? and endtime>=?";
  } else {
    sql = "SELECT filename FROM tsindex WHERE network=? and station=? and "
          "location=? and channel=?";
  }
  rc = sqlite3_prepare_v2(pdb, sql, -1, &select_stmt, NULL);
  if (SQLITE_OK != rc) {
    logit("e", "Can't prepare statment %s (%i): %s\n", sql, rc,
          sqlite3_errmsg(pdb));
    sqlite3_close(pdb);
    return rc;
  }

  rc = msdb_bind_text(select_stmt, 1, net);
  if (rc != SQLITE_OK) {
    logit("e", "Error binding value in statment (%i): %s\n", rc,
          sqlite3_errmsg(pdb));
    msdb_close(pdb, select_stmt);
    return rc;
  }
  rc = msdb_bind_text(select_stmt, 2, stat);
  if (rc != SQLITE_OK) {
    logit("e", "Error binding value in statment (%i): %s\n", rc,
          sqlite3_errmsg(pdb));
    msdb_close(pdb, select_stmt);
    return rc;
  }
  rc = msdb_bind_text(select_stmt, 3, loc);
  if (rc != SQLITE_OK) {
    logit("e", "Error binding value in statment (%i): %s\n", rc,
          sqlite3_errmsg(pdb));
    msdb_close(pdb, select_stmt);
    return rc;
  }
  rc = msdb_bind_text(select_stmt, 4, chan);
  if (rc != SQLITE_OK) {
    logit("e", "Error binding value in statment (%i): %s\n", rc,
          sqlite3_errmsg(pdb));
    msdb_close(pdb, select_stmt);
    return rc;
  }
  if (t > 0) {
    datestriso(ceil(t), start_timestr, sizeof(start_timestr));
    rc = msdb_bind_text(select_stmt, 5, start_timestr);
    if (rc != SQLITE_OK) {
      logit("e", "Error binding value in statment (%i): %s\n", rc,
            sqlite3_errmsg(pdb));
      msdb_close(pdb, select_stmt);
      return rc;
    }
    datestriso(floor(t), end_timestr, sizeof(end_timestr));
    rc = msdb_bind_text(select_stmt, 6, end_timestr);
    if (rc != SQLITE_OK) {
      logit("e", "Error binding value in statment (%i): %s\n", rc,
            sqlite3_errmsg(pdb));
      msdb_close(pdb, select_stmt);
      return rc;
    }
  }

  rc = sqlite3_step(select_stmt);
  if (SQLITE_ROW == rc) {
    const char *str = (const char *)sqlite3_column_text(select_stmt, 0);
    strncpy(mseedfile, str, MSEEDFILE_SZ);
    rc = sqlite3_step(select_stmt);
  }

  if (SQLITE_DONE != rc) {
    logit("e", "select statement didn't return DONE (%i): %s\n", rc,
          sqlite3_errmsg(pdb));
  }

  msdb_close(pdb, select_stmt);
  return rc;
}
