#define __GM_DATA_C__
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <earthworm.h>
#include "config.h"
#include "gm_data.h"

static int largest_seconds = 0;
static int data_table_size = 0;

int decrement_data_index(int *index)
{
   if (--*index < 0)
   {
      *index = data_table_size - 1;
   }
   return *index;
}

int increment_data_index(int *index)
{
   if (++*index >= data_table_size)
   {
      *index = 0;
   }
   return *index;
}

void get_largest_pg_values(PG_TABLE_ENTRY *largest_pg_values, time_t timeNow, int time_span, STATION *Sta)
{
   double time, value;
   int i;
   int index = Sta->last_saved_index;
   int numIntervals = time_span / Gparm.OutputInterval;
   double minTime = 0.0;
   memset(largest_pg_values, 0, sizeof(PG_TABLE_ENTRY));
   if (Gparm.useCurrentTimeForWindow) {
      minTime = timeNow - time_span;
   }
   for (i = 0; i < numIntervals; i++)
   {
      if (i != 0) {
         decrement_data_index(&index);
      }
      time = Sta->pg_table[index].pga.time;
      // if value is not too old
      if (time >= minTime) {
         value = Sta->pg_table[index].pga.value;
         if (value > largest_pg_values->pga.value)
         {
            largest_pg_values->pga.value = value;
            largest_pg_values->pga.time = time;
            strcpy(largest_pg_values->pga.chan, Sta->pg_table[index].pga.chan);
         }
      }
      time = Sta->pg_table[index].pgv.time;
      // if value is not too old
      if (time >= minTime) {
         value = Sta->pg_table[index].pgv.value;
         if (value > largest_pg_values->pgv.value)
         {
            largest_pg_values->pgv.value = value;
            largest_pg_values->pgv.time = time;
            strcpy(largest_pg_values->pgv.chan, Sta->pg_table[index].pgv.chan);
         }
      }
   }
}

int init_gm_data()
{
   if (data_table_size == 0)
   {
      // determine the larget window
      int index;
      int seconds;
      for (index = 0; index < Gparm.NumOutputWindows; index++)
      {
         seconds = Gparm.OutputWindows[index];
         if (seconds > largest_seconds)
         {
            largest_seconds = seconds;
         }
      }
      data_table_size = largest_seconds / Gparm.OutputInterval;
      if (Gparm.Debug)
      {
         printf("report_gm: Gparm.OutputInterval=%d\n", Gparm.OutputInterval);
         printf("report_gm: largest_seconds=%d\n", largest_seconds);
         printf("report_gm: data_table_size=%d\n", data_table_size);
      }
   }
   return data_table_size;
}

void save_values(time_t timeNow, STATION *Sta, char* chan, double time, double pga, double pgv)
{
   int current_index = (((time_t)time) / Gparm.OutputInterval) % data_table_size;
   // if same window as last time
   if (current_index == Sta->last_saved_index)
   {
      // udpdate value if larger
      if (pga > Sta->pg_table[current_index].pga.value)
      {
         Sta->pg_table[current_index].pga.value = pga;
         Sta->pg_table[current_index].pga.time = time;
         strcpy(Sta->pg_table[current_index].pga.chan, chan);
         if (Gparm.Debug)
         {
            printf("report_gm: save pga value(%s.%s.%s.%s, %.12g, %d, %.12g)\n", 
                   Sta->sta, chan, Sta->net, Sta->loc, time, current_index, pga);
         }
      }
      if (pgv > Sta->pg_table[current_index].pgv.value)
      {
         Sta->pg_table[current_index].pgv.value = pgv;
         Sta->pg_table[current_index].pgv.time = time;
         strcpy(Sta->pg_table[current_index].pgv.chan, chan);
         if (Gparm.Debug)
         {
            printf("report_gm: save pgv value(%s.%s.%s.%s, %.12g, %d, %.12g)\n", 
                   Sta->sta, chan, Sta->net, Sta->loc, time, current_index, pgv);
         }
      }
   }
   else
   {
      // if not first value saved and not using current time
      // it is not necessary to clear if checking the time
      if (Sta->last_saved_index != -1 && !Gparm.useCurrentTimeForWindow)
      {
         // clear previous table values if gap
         int index = Sta->last_saved_index;;
         while ((index = increment_data_index(&index)) != current_index)
         {
            if (Gparm.Debug)
            {
               printf("report_gm: clearing(%s.%s.%s.%s, %d)\n", 
                      Sta->sta, chan, Sta->net, Sta->loc, index);
            }
            memset(Sta->pg_table + current_index, 0, sizeof(PG_TABLE_ENTRY));
         }
      }
      if (Gparm.Debug)
      {
         printf("report_gm: save_values(%s.%s.%s.%s, %.10g, %d, %.12g, %.12g)\n", 
                Sta->sta, chan, Sta->net, Sta->loc, time, current_index, pga, pgv);
      }
      Sta->pg_table[current_index].pga.value = pga;
      Sta->pg_table[current_index].pga.time = time;
      Sta->pg_table[current_index].pgv.value = pgv;
      Sta->pg_table[current_index].pgv.time = time;
      strcpy(Sta->pg_table[current_index].pgv.chan, chan);
      if ( time > Sta->pg_table[current_index].pga.time)
      {
         Sta->pg_table[current_index].pga.time = time;
      }
      if ( time > Sta->pg_table[current_index].pgv.time)
      {
         Sta->pg_table[current_index].pgv.time = time;
      }
      Sta->last_saved_index = current_index;
   }
   Sta->time = time;
   Sta->latency = timeNow - time;
}
