#include "fdsn_client.h"
#include "strfuncs.h"
#include "time_ew.h"
#include <stdio.h>
#include <string.h>

static void append(char *program, char *str) {
  int len = strlen(program);
  strncpy(program + len, str, PROGRAM_SIZE - len);
}

static void appendparam(char *program, char *paramname, char *paramvalue) {
  if (paramvalue != NULL && *paramvalue) {
    int len = strlen(program);
    snprintf(program + len, PROGRAM_SIZE - len, "%s=%s&", paramname,
             paramvalue);
  }
}

/**
 * Clear the FDSN client information.
 * @param fci the FDSN client information
 */
void fdsn_client_clear(FDSN_CLIENT_INFO *fci) {
  memset(fci, 0, sizeof(FDSN_CLIENT_INFO));
  strcpy(fci->loc, EMPTY_LOC);
}

/**
 * Get the program path for the FDSN client (fci->program). The fci->wgetpath,
 * fci->wsurl and fci->service values must be set.
 * @param fci the FDSN client information
 */
void fdsn_client_get(FDSN_CLIENT_INFO *fci) {
  char *program = fci->program;
  *program = '\0';
  if (!*fci->wgetpath || !*fci->wsurl || !*fci->service) {
    return;
  }
  append(program, fci->wgetpath);
  append(program, " ");
  append(program, fci->wsurl);
  if (!strendswith(fci->wsurl, "/")) {
    append(program, "/");
  }
  append(program, fci->service);
  append(program, "/1/query?");
  appendparam(program, "net", fci->net);
  appendparam(program, "sta", fci->sta);
  appendparam(program, "loc", fci->loc);
  appendparam(program, "cha", fci->chan);
  appendparam(program, "starttime", fci->starttime);
  appendparam(program, "endtime", fci->endtime);
  appendparam(program, "format", fci->format);
  append(program, "nodata=404");
}

/**
 * Set the end time.
 * @param fci the FDSN client information
 * @param t the eoch for the end time
 */
void fdsn_client_set_endtime(FDSN_CLIENT_INFO *fci, double t) {
  datestriso(t, fci->endtime, sizeof(fci->endtime));
}

/**
 * Set the start time.
 * @param fci the FDSN client information
 * @param t the eoch for the start time
 */
void fdsn_client_set_starttime(FDSN_CLIENT_INFO *fci, double t) {
  datestriso(t, fci->starttime, sizeof(fci->starttime));
}
