import os
import socket
import sys

#import warnings
#logging.getLogger('matplotlib').setLevel(logging.WARNING)
#import matplotlib
#import matplotlib.pyplot as plt
#plt.ioff()

from flask import abort, redirect, render_template, send_file, send_from_directory, url_for
from flask import session, request
#from flask_restful import reqparse, Api, Resource

from obspy import read
from obspy.clients.earthworm import Client
from obspy.core.utcdatetime import UTCDateTime

from ewave_plot.app import app
#from ewave_plot.app.forms import DataRequestForm
from ewave_plot.libs.waveform import WaveformPlotting

WSV_HOSTS = app.config['WSV_HOSTS']
static_folder = app.config['STATIC_FOLDER']

logger = app.logger

from ewave_plot.app.query import init_query_parser
parser = init_query_parser()


# Required fields in form:
form_fields = ['net', 'sta', 'cha', 'loc',
               'start_date', 'hh', 'mm', 'ss', 'dur',
               ]

# Default values for form fields:
default_values = {
    'net':'', 'sta':'', 'cha':'', 'loc':'',
    'dur':'', 'start_date':'',
    'hh':'', 'mm':'', 'ss':'',
    'outputFormat': 'png',
    'displayMaxValue': False,
    'scaleFactor': '', 'units':'',
    'forceReplot': False,
    'plabel': '',
    'p_date': '',
    'p_hh': '', 'p_ss': '', 'p_mm': '',
}

checkboxes = ['displayMaxValue', 'forceReplot']

@app.route('/ewave/form', methods=['GET', 'POST'])
def processForm():
    if request.method == 'GET':
        pass
        #print("Got method=GET")
        #modify_session(clear=True)
    elif request.method == 'POST':
        #print("Got method=POST action=%s" % request.form['action'])

        clear = True if 'action' in request.form and request.form['action'] == 'clear' else False

        if clear:
            modify_session(clear=True)
        else:
            invalid = check_form()
            modify_session()

            if not invalid:
                form = request.form
                """
                plabel, ptime
                """
                date_str = '%sT%s:%s:%s' % (form['start_date'], form['hh'], form['mm'], form['ss'])
                start = UTCDateTime(date_str)
                #for k,v in form.items():
                    #print("processForm form[%s]=%s" % (k,v))

                string = ('/ewave/query?net={}&sta={}&cha={}&loc={}'
                          '&start={}&dur={}&format={}')\
                         .format(form['net'], form['sta'], form['cha'], form['loc'],
                                 start, form['dur'], form['outputFormat'])

                for field in ['scaleFactor', 'units', 'displayMaxValue', 'forceReplot']:
                    if field in checkboxes:
                        if form.get(field):
                            string += '&{}={}'.format(field, True)
                    elif form[field].strip():
                        string += '&{}={}'.format(field, form[field])

                if form['p_date'] and form['p_hh'] and form['p_mm'] and form['p_ss']:
                    date_str = '%sT%s:%s:%s' % (form['p_date'], form['p_hh'], form['p_mm'], form['p_ss'])
                    ptime = UTCDateTime(date_str)
                    plabel = form['plabel'] if form['plabel'].strip() else "P"
                    string += '&{}={}&{}={}'.format('ptime', ptime, 'plabel', plabel)

                logger.info("processForm: redirect url=[%s]" % string)
                return redirect(string)

    return render_template('form.html', session=session)


from flask import flash

def check_form():
    '''
    check that required fields are filled out in form
    if not --> flash error message(s) to form
    '''

    invalid = False

    for field in form_fields:
        value = request.form[field]
        if value is None or value == '':
            error = "*Please enter a valid %s" % field
            flash(error)
            invalid = True

    return invalid


def modify_session(clear=False):
    '''
    modify session dict with current form state
    '''

    for key in default_values.keys():
        if clear:
            session[key] = default_values[key]
        else:
            if key in checkboxes:
                session[key] = request.form.get(key)
            else:
                session[key] = request.form[key]
    return


@app.route('/ewave/query', methods=['GET', 'POST'])
def GetRESTData():
    fname = 'GetRESTData'

    logger.info("Process %s" % (request))
    logger.info("path=[%s] query_string=[%s]" % (request.path, request.query_string.decode('utf-8')))

    # MTH: This POS doesn't even work!! returns error msgs in e.data but throws empty abort page!
    """
    import flask_restful
    #errors = {'start': 'Valid start time required', 'dur': 'Duration must be non-zero integer value'}
    try:
        flask_restful.abort(404, message=errors)
    except Exception as e:
        print("abort threw exception:%s" % e)
        print(e.data)
    """

    error = None
    args= None
    # MTH: parser_args will throw an exception if required arg like
    #      start is missing or if dur=61a is not an int
    #      However, it will do nothing in the case of unknown args
    #      Instead we pass those errors back as error string
    try:
        args, error = parser.parse_args()
    except Exception as e:
        #print("Caught exception from parse_args():%s" % e)
        logger.error("parse_args() Caught exception:%s" % e)
        msg = ''
        for k,v in e.data['message'].items():
            msg += "%s: %s\n" % (k,v)
        #print(e.data)
        abort(400, msg)

    if error:
        abort(400, error)
        logger.error("caught error=%s" % (error))
        return render_template('help.html')

    if args is None:
        logger.error("args is None!")
        return render_template('help.html')

    #for key in default_values.keys():
        #print("GetREST: args[%s]=%s" % (key, getattr(args, key, None)))

    network = args.net
    station = args.sta
    #if not args.loc:
        #location = '--'
    #else:
        #location = args.loc
    location = '--' if not args.loc else args.loc

    channel = args.cha
    starttime = UTCDateTime(args.start)
    if args.end:
        endtime = UTCDateTime(args.end)
        logger.info("Use endtime=%s" % endtime)
    else:
        #endtime = UTCDateTime(starttime + int(args.dur))
        endtime = UTCDateTime(starttime + float(args.dur))
        logger.info("Dur=%s --> Calc endtime=%s" % (args.dur, endtime))

    outputFormat = 'png'   # Default

    if args.format:
        if args.format in ['plot', 'png']:
            outputFormat = 'png'
        elif args.format in ['miniseed', 'mseed']:
            outputFormat = 'mseed'
        else:
            # these are the only supported output types at this time!
            abort(404)

    # First, check the static/ directory to see if we have already acquired
    # this data before:
    outputFileName = '{}.{}.{}.{}_{}-{}.{}'.format(network, station,
                                                   location, channel,
                                                   starttime, endtime,
                                                   outputFormat)

    outfile = os.path.join(static_folder, outputFileName)


    pltname = '{}.{}.{}.{}_{}-{}.{}'.format(network, station,
                                            location, channel,
                                            starttime, endtime,
                                            'png')
    pltname = os.path.join(static_folder, pltname)
    mseedname = pltname.replace('.png', '.mseed')

    if args.forceReplot:
        logger.info("GetREST: forceReplot ==> ignore possible existing plot")
    else:
        if os.path.exists(outfile):
            logger.info("File:%s already exists in static_folder --> return it!" % outputFileName)
            #print("File:%s already exists in static_folder --> return it!" % outputFileName)
            return app.send_static_file(outputFileName)

    if os.path.exists(mseedname):
        logger.info("mseedname=[%s] already exists --> use this!" % mseedname)
        #print("mseedname=[%s] already exists --> use this!" % mseedname)
        st = read(mseedname)
    else:
        logger.info("get_waveform_data for mseedname=[%s]" % mseedname)
        st = get_waveform_data(network, station, location, channel, starttime, endtime)
        #print("get_waveform returned st len:%d" % len(st))
        demean = True
        #demean = False
        if demean:
            st.detrend(type='demean')

    st.write(mseedname, format="MSEED")

    if outputFormat == 'mseed':
        # write this out
        logger.info("Write mseed to file:%s" % outputFileName)
        return send_from_directory(static_folder, outputFileName, as_attachment=True)

    # Plot the waveform(s) and return png
    else:
        kwargs = {}
        kwargs['outfile'] = outfile

        picktime = args.ptime
        picklabel = args.plabel
        if picktime:
            if picktime == '':
                picktime = None
            if not picklabel:
                picklabel = 'Pick'
            kwargs['picktime'] = picktime
            kwargs['picklabel'] = picklabel

        displayMax = args.displayMaxValue
        scaleFactor = args.scaleFactor
        outputUnits = args.units
        #print("displayMax=%s scaleFactor=%s" % (displayMax, scaleFactor))
        if displayMax:
            if scaleFactor is None or scaleFactor <= 0.0:
                scaleFactor = 1.0
            if outputUnits is None:
                outputUnits = 'counts'
            kwargs['displayMaxValue'] = True
            kwargs['scaleFactor'] = scaleFactor
            kwargs['units'] = outputUnits

            if scaleFactor != 1.0:
                for tr in st:
                    tr.data *= scaleFactor

        logger.info("Save plot to file:%s" % outputFileName)
        plotData(st, **kwargs)
        #print("return outputFileName:%s" % outputFileName)
        return app.send_static_file(outputFileName)

    # End of get()
# End of class

@app.route('/')
@app.route('/ewave')
@app.route('/ewave/')
@app.route('/ewave/usage')
@app.route('/ewave/help')
def return_usage():
    #print("Enter return_usage()")
    return render_template('help.html')
    #return app.send_static_file(helpfile)


@app.route('/ewave/availability')
@app.route('/ewave/data')
def get_data_availability():

    all_available = []
    error_codes = []
    for hst, prt in WSV_HOSTS:
        #hst = wsv[0]
        #prt = wsv[1]

        # check if we can make a connection to (addr, port) before
        # setting up client:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(5)
        logger.info("check socket connection to hst:%s prt:%s" % (hst, prt))
        #print("check socket connection to hst:%s prt:%s" % (hst, prt))
        errNo = None
        try:
            errNo = s.connect_ex((hst, prt))
        except Exception as e:
            logger.error("Testing socket connect to %s:%s ==> Caught exception:%s" %
                         (hst, prt, e))
            error_codes.append("Testing socket connect to %s:%s ==> Caught exception:%s" %
                              (hst, prt, e))
        else:
            s.close()

        if errNo !=0:
            error_codes.append("check socket connection to hst:%s prt:%s returned errNo:%s" %
                               (hst, prt, errNo))
            logger.error("check socket connection to hst:%s prt:%s returned errNo:%s" %
                               (hst, prt, errNo))
            continue

        client = Client(hst, prt)
        # MTH: only channel code can be wildcarded!
        # If you don't match any scnl tanks availability will be empty list
        availability = client.get_availability('*', '*', '*', '*')
        availability = [a + (hst,prt) for a in availability]
        if availability:

            #print("%5s %5s %5s %5s %30s %30s %s:%s" % ("Net", "Sta", "Loc", "Cha", "Start", "End", "Host", "Port"))
            #for elem in sorted(availability, key=lambda x: (x[0], x[1], x[2], x[3], x[4])):
                #('X1', 'SA17', '00', 'HHZ', UTCDateTime(2021, 11, 29, 15, 59, 14, 150000), UTCDateTime(2021, 11, 30, 22, 16, 26, 630000))
                #print(elem)
                #print("%5s %5s %5s %5s %30s %30s %s:%s" % elem)
            all_available.append(sorted(availability, key=lambda x: (x[0], x[1], x[2], x[3], x[4])))
        else:
            logger.warning("No Data found at %s:%s" % (hst, prt))

    if all_available:
        return render_template('availability.html', availability=all_available)
    else:
        error_codes.append("No Available Data !!")
        abort(500, "\n".join(error_codes))



def socket_worked(hst, prt):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(5)
    logger.info("check socket connection to hst:%s prt:%s" % (hst, prt))
    #print("socket_worked: check socket connection to hst:%s prt:%s" % (hst, prt))
    errNo = None
    try:
        errNo = s.connect_ex((hst, prt))
    except Exception as e:
        logger.error("Testing socket connect to %s:%s ==> Caught exception:%s" %
                     (hst, prt, e))
        return False
    else:
        s.close()

    if errNo !=0:
        #print("socket_worked: errNo=%d --> return False" % errNo)
        logger.warning("socket_worked: errNo=%d --> return False" % errNo)
        return False

    #print("socket_worked: return True")
    return True

def get_waveform_data(network, station, location, channel, starttime, endtime):
        st = None
        i = 0
        for hst, prt in WSV_HOSTS:
            logger.info("Check for data at: [%s:%s] network:%s station:%s channel:%s location:%s starttime:%s endtime:%s" %
                        (hst, prt, network, station, channel, location, starttime, endtime))

            logger.debug("get_waveform_data: Call socket_worked(hst=%s prt=%s)" % (hst, prt))
            if socket_worked(hst, prt):
                logger.debug("get_waveform_data: socket_worked(hst=%s prt=%s) --> call Client" % (hst, prt))
                client = Client(hst, prt)
                if client.get_availability(network, station, location, channel):
                    logger.info("Client(%s,%s) HAS availability for nscl:%s.%s.%s.%s" %
                                (hst, prt, network, station, channel, location))
                    # You can get to here but if starttime/endtime are out of the wsv_tnk bounds, get_waveforms
                    #   will throw an error - Does it get the other channels ??
                    logger.debug("Call client.get_waveforms(%s,%s,%s,%s,%s,%s)" %
                                (network, station, location, channel, starttime, endtime))
                    st = client.get_waveforms(network, station, location, channel, starttime, endtime)
                    #logger.debug("client.get_waveforms(%s,%s,%s,%s,%s,%s) returned st.traces len:%s" %
                                #(network, station, location, channel, starttime, endtime, len(st.traces)))
                    if st:
                        logger.debug(st[0].stats)
                        #print("st is something")
                        break
                    else:
                        logger.warning("client.get_waveforms(%s,%s,%s,%s,%s,%s) ==> returned st=None !" %
                                (network, station, location, channel, starttime, endtime))
                else:
                    logger.warning("Client(%s,%s) has NO availability for nscl:%s.%s.%s.%s" %
                                (hst, prt, network, station, channel, location))


        if st is None or st.count() == 0:
            msg = "No Data found in wave_servers for: %s.%s.%s.%s start:%s" % \
                    (network,station,channel,location,starttime)
            abort(404, msg)
        else:
            #print("Here call st.merge")
            return st.merge()


def plotData(stream, **kwargs):
    logger.info("plotData")
    '''
        #waveform = WaveformPlotting(stream=obspyStream, *args, **kwargs)
        #waveform.plot_waveform(*args, **kwargs)
        #waveform = WaveformPlotting(stream=obspyStream, outfile=outfile)
    '''
    waveform = WaveformPlotting(stream=stream, **kwargs)
    waveform.plot_waveform(**kwargs)

    return


#api.add_resource(GetRESTData, '/ewave/<request_string>')


#def main():
    #app.run(debug=False, host=FLASK_HOST, port=FLASK_PORT)

if __name__ == '__main__':
    main()
    #app.run(debug=False, host=FLASK_HOST, port=FLASK_PORT)
